;***************************************************************************
; bit_fifo.asm
;***************************************************************************

; 18. March 2003
; Burkhard John
; Harlessstr. 9
; 40239 Duesseldorf
; email: burkhard-john@online-club.de

; buffer functions:

; bit_fifo_init
; bit_fifo_get_entry
; bit_fifo_write_entry
; bit_fifo_empty
; bit_fifo_full

;buffer--------------------------------------------------------------------
                ; memory usage
; in SRAM
;buffer:
;        .byte   length_of_buffer

; registers
;
; .def	data	=	r24     ; must be defined in main program, 
;				  different form following registers
;				  registers must not be used by interrupts
;				  without push/pop

.def	bit_fifo_XB	=	r23
.def	bit_fifo_YB	=	r22
.def	bit_fifo_inc	=	r21
.def	bit_fifo_zero	=	r20
.def	bit_fifo_help	=	r19
	
; usable space is length_of_buffer - 8, so minimum length of buffer is 10
; for 2 bytes buffer. Maximum length is limited
; only by size of SRAM.

.equ    bit_fifo_length_l            =    0
.equ    bit_fifo_length_h            =    1
.equ    bit_fifo_in_pointer_b        =    2
.equ    bit_fifo_in_pointer_l        =    3
.equ    bit_fifo_in_pointer_h        =    4
.equ    bit_fifo_out_pointer_b       =    5
.equ    bit_fifo_out_pointer_l       =    6
.equ    bit_fifo_out_pointer_h       =    7

.equ    length_of_bit_fifo_descriptor     =       8

.equ	bit_fifo_increment           =    32

;buffer--------------------------------------------------------------------
                                ; initializes a byte-buffer
                                ; base address in ZH:ZL
                                ; length of buffer in YH:YL
bit_fifo_init:
	push	data
	push	YL
	push	YH
	
        sbiw    YL,length_of_bit_fifo_descriptor
        std     Z+bit_fifo_length_l,YL
        std     Z+bit_fifo_length_h,YH
	ldi	data,0
        mov     YL,ZL
        mov     YH,ZH
        adiw    YL,length_of_bit_fifo_descriptor
	std	Z+bit_fifo_out_pointer_b,data
        std     Z+bit_fifo_out_pointer_l,YL
        std     Z+bit_fifo_out_pointer_h,YH
        ldi	data,bit_fifo_increment
	std	Z+bit_fifo_in_pointer_b,data
        std     Z+bit_fifo_in_pointer_l,YL
        std     Z+bit_fifo_in_pointer_h,YH

	pop	YH
	pop	YL
	pop	data
        ret

;buffer--------------------------------------------------------------------
                        ; gets one byte from buffer in register data
                        ; basepointer of buffer in ZH:ZL
                        ; must not be called if buffer is empty, check first
bit_fifo_get_entry:
;        cli
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH
	push	bit_fifo_inc
	push	bit_fifo_zero
                                        ; calculate new out pointer
        mov     XL,ZL
        mov     XH,ZH
        adiw    XL,length_of_bit_fifo_descriptor
        ldd     YL,Z+bit_fifo_length_l
        ldd     YH,Z+bit_fifo_length_h
        add     XL,YL
        adc     XH,YH

	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

	ldi	bit_fifo_inc,bit_fifo_increment
	clr	bit_fifo_zero

	add	bit_fifo_YB,bit_fifo_inc
	adc	YL,bit_fifo_zero
	adc	YH,bit_fifo_zero
                                        ; if YH:YL out of buffer
                                        ; then set YH:YL to begin of buffer
        sub     XL,YL
        sbc     XH,YH
        brne    bit_fifo_get_entry_get

        mov     YL,ZL
        mov     YH,ZH
        adiw    YL,length_of_bit_fifo_descriptor
	clr	bit_fifo_YB

bit_fifo_get_entry_get:
                                        ; save new out pointer
	std	Z+bit_fifo_out_pointer_b,bit_fifo_YB
        std     Z+bit_fifo_out_pointer_l,YL
        std     Z+bit_fifo_out_pointer_h,YH

                                        ; get entry from buffer
        ld      data,Y
	ldd	bit_fifo_inc,Z+bit_fifo_out_pointer_b
	ldi	bit_fifo_zero,1

bit_fifo_get_shift_loop:
	tst	bit_fifo_inc
	breq	bit_fifo_get_shifted

	lsl	bit_fifo_zero
	subi	bit_fifo_inc,bit_fifo_increment
	rjmp	bit_fifo_get_shift_loop
	
bit_fifo_get_shifted:
	and	data,bit_fifo_zero
	breq	bit_fifo_get_end

	ldi	data,1

bit_fifo_get_end:	
	pop	bit_fifo_zero
	pop	bit_fifo_inc
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
;        sei
        ret

;buffer--------------------------------------------------------------------
                                ; writes one byte in register data to buffer
                                ; base address in ZH:ZL
                                ; if buffer is full, entry is ignored
                                ; no other register is modified
bit_fifo_write_entry:
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH
	push	bit_fifo_inc
	push	bit_fifo_zero

	ldd	bit_fifo_XB,Z+bit_fifo_in_pointer_b
        ldd     XL,Z+bit_fifo_in_pointer_l
        ldd     XH,Z+bit_fifo_in_pointer_h
	
	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

	sub	bit_fifo_YB,bit_fifo_XB
        sbc     YL,XL
        sbc     YH,XH
                                ; if buffer full, in_pointer = out_pointer
                                ; then ignore character
        breq    bit_fifo_write_entry_end

	ldi	bit_fifo_zero,1
	ldd	bit_fifo_inc,Z+bit_fifo_in_pointer_b
	
bit_fifo_write_shift_loop:
	tst	bit_fifo_inc
	breq	bit_fifo_write_shifted

	lsl	bit_fifo_zero
	subi	bit_fifo_inc,bit_fifo_increment
	rjmp	bit_fifo_write_shift_loop
	
bit_fifo_write_shifted:	
	ld	bit_fifo_inc,X
	tst	data
	brne	bit_fifo_write_1

	com	bit_fifo_zero
	and	bit_fifo_inc,bit_fifo_zero
	rjmp	bit_fifo_write
	
bit_fifo_write_1:	
	or	bit_fifo_inc,bit_fifo_zero

bit_fifo_write:	
	st	X,bit_fifo_inc

	ldi	bit_fifo_inc,bit_fifo_increment
	clr	bit_fifo_zero

	add	bit_fifo_XB,bit_fifo_inc
	adc	XL,bit_fifo_zero
	adc	XH,bit_fifo_zero
	
                                ; check for input pointer out of buffer
        ldd     YL,Z+bit_fifo_length_l
        ldd     YH,Z+bit_fifo_length_h
        adiw    YL,length_of_bit_fifo_descriptor
        add     YL,ZL
        adc     YH,ZH
        sub     YL,XL
        sbc     YH,XH
        brne    bit_fifo_write_pointer_ok
                        ; if pointer out of buffer, set it to begin of buffer
        mov     XL,ZL
        mov     XH,ZH
        adiw    XL,length_of_bit_fifo_descriptor
	clr	bit_fifo_XB

bit_fifo_write_pointer_ok:
                                ; save new input pointer
	std	Z+bit_fifo_in_pointer_b,bit_fifo_XB
        std     Z+bit_fifo_in_pointer_l,XL
        std     Z+bit_fifo_in_pointer_h,XH

bit_fifo_write_entry_end:
	pop	bit_fifo_zero
	pop	bit_fifo_inc
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
        ret

;buffer--------------------------------------------------------------------
                                ; if buffer is empty, register data is cleard
                                ; else register data is set
                                ; base pointer in ZH:ZL
bit_fifo_empty:
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH
	push	bit_fifo_inc
	push	bit_fifo_zero

	ldi	bit_fifo_inc,bit_fifo_increment
	clr	bit_fifo_zero

        ser     data

	ldd	bit_fifo_XB,Z+bit_fifo_in_pointer_b
        ldd     XL,Z+bit_fifo_in_pointer_l
        ldd     XH,Z+bit_fifo_in_pointer_h
	
	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

        add	bit_fifo_YB,bit_fifo_inc
	adc	YL,bit_fifo_zero
	adc	YH,bit_fifo_zero

	sub	bit_fifo_XB,bit_fifo_YB
        sbc     XL,YL
        sbc     XH,YH
                        ; buffer empty if (in_pointer - (out_pointer+1))=0
        breq    bit_fifo_is_empty

	ldd	bit_fifo_XB,Z+bit_fifo_in_pointer_b
        ldd     XL,Z+bit_fifo_in_pointer_l
        ldd     XH,Z+bit_fifo_in_pointer_h
	
	sub	bit_fifo_YB,bit_fifo_XB
        sbc     YL,XL
        sbc     YH,XH

	clr	bit_fifo_XB
        ldd     XL,Z+bit_fifo_length_l
        ldd     XH,Z+bit_fifo_length_h
	
	sub	bit_fifo_XB,bit_fifo_YB
        sbc     XL,YL
        sbc     XH,YH
                                        ; or buffer empty if
                                        ; pointers at opposite ends of buffer
                                        ; out_pointer+1 - in_pointer
                                        ; -length_of_buffer = 0
        brne    bit_fifo_empty_end

bit_fifo_is_empty:
        clr     data

bit_fifo_empty_end:
	tst	data
	pop	bit_fifo_zero
	pop	bit_fifo_inc
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
        ret

;buffer--------------------------------------------------------------------
                                ; if buffer is full, register data is cleard
                                ; else register data is set
                                ; base pointer in ZH:ZL
bit_fifo_full:
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH

        ser     data

	ldd	bit_fifo_XB,Z+bit_fifo_in_pointer_b
        ldd     XL,Z+bit_fifo_in_pointer_l
        ldd     XH,Z+bit_fifo_in_pointer_h
	
	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

	sub	bit_fifo_XB,bit_fifo_YB
        sbc     XL,YL
        sbc     XH,YH
                                ; buffer is full if in_pointer = out_pointer
        brne    bit_fifo_full_end

bit_fifo_is_full:
        clr     data

bit_fifo_full_end:
	tst	data
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
        ret

;*****************************************************************************
