
;*****************************************************************
; multiply_n_n_to_2n.asm
;*****************************************************************
;
; Burkhard John
; Harlesstr. 9
; D-40239 Duesseldorf
; burkhard-john@online-club.de
;
; 6. Mar. 2004
;
;*****************************************************************
;
; This is free software, you may use it for any purpose.
; Use it on your own risk!
;
;*****************************************************************
;
; multiplication of two n bit numbers to 2n number, unsigned.
;
; n=1..127 make shure that all bytes have space in RAM, see below.
;
; requires MegaAVRs for multiplication
;
; multiplication data are kept on stack, all used registers are saved.
;
;
	;; ++++++++++++++++++++++++++++++++++++++++++++++

	;; usage pre
	;; push multiplicator_LSB
	;;    .
	;;    n-2 pushes
	;;    .
	;; push multiplicator_MSB
	;; push multiplicand_LSB
	;;    .
	;;    n-2 pushes
	;;    .
	;; push multiplicand_MSB
	;; push n

	;; rcall multiply_n_n_2n
	
	;; usage post
	;; pop n----------dummy
	;; pop result_MSB
	;;    .
	;;    2n-2 pops
	;;    .
	;; pop result_LSB
	
	;; ++++++++++++++++++++++++++++++++++++++++++++++

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; data_space 
;
;-----------------------------------------------------------------
;
; registers
; X,Y,Z, R0, R1 must be used, any other register (R2...R25) may be used.
; Registers must not be used by interrupts without push/pop !!!!!!!
	
.def	multiply_l		=	R0 ; for mul instruction
.def	multiply_h		=	R1 ; for mul instruction
	
.def	multiply_digits		=	r2
.def	multiply_counter	=	r3
.def	multiply_data		=	r4
.def	multiply_mult		=	r5
.def	multiply_inner_counter	=	r25
	
;					Pointers
;				XL 
;				XH
;				YL
;				YH
;				ZL
;				ZH


;stack usage
;
;  pre
;
; stack top
; multiplicator LSB
;   .
;   n-2 bytes
;   .
; multiplicator MSB
; multiplicand LSB
;   .
;   n-2 bytes
;   .
; multiplicand MSB
; number of bytes to multiply
; new stack top
	
;
;  post
;
; result LSB
;   .
;   2n-2 bytes
;   .
; result MSB
; number of bytes to multiply, now dummy

	
; bytes used on stack:	
; 2*n(parameter) +1(number) +2(call) +13(push) +2*n(internal result) +4(push)

; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
multiply_n_n_2n:
	
.equ	multiply_start_of_pushes	= PC - 2 ; 2 for call
	
	push	ZL
	push	ZH
	push	YL
	push	YH
	push	XL
	push	XH
	push	multiply_l
	push	multiply_h
	push	multiply_digits
	push	multiply_counter
	push	multiply_inner_counter
	push	multiply_data
	push	multiply_mult

.equ	multiply_number_of_pushes	= PC - multiply_start_of_pushes

	clr	multiply_data
	in	ZL,SPL
	in	ZH,SPH
	;;			Z: pointer to LSB of result on stack
	ld	multiply_digits,Z+multiply_number_of_pushes+1
	ldi	YL,LOW(multiply_number_of_pushes+1)
	ldi	YH,HIGH(multiply_number_of_pushes+1)
	add	YL,ZL
	adc	YH,ZH
	add	YL,multiply_digits
	adc	YH,multiply_data
	;;			Y: pointer to LSB of multiplicand on stack
	
	movw	XL,YL
	add	XL,multiply_digits
	adc	XH,multiply_data
	;;			X: pointer to LSB of multiplicator on stack

	mov	multiply_counter,multiply_digits
	lsl	multiply_counter ; *2
;	clr	multiply_data
	
multiply_create_zero_result:	
	push	multiply_data
	dec	multiply_counter
	brne	multiply_create_zero_result
	
	mov	multiply_counter,multiply_digits
	
;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	
multiply_outer_loop:
	push	YL
	push	YH
	push	ZL
	push	ZH

	mov	multiply_inner_counter,multiply_digits
	ld	multiply_data,X
	
;..............................................................
	
multiply_inner_loop:
	ld	multiply_mult,Y
	mul	multiply_data,multiply_mult
	ld	multiply_mult,Z
	add	multiply_mult,multiply_l
	st	Z,multiply_mult
	in	multiply_mult,SREG ; save carry flag
	sbiw	Z,1
	out	SREG,multiply_mult ;  restore carry flag
	ld	multiply_mult,Z
	adc	multiply_mult,multiply_h
	st	Z,multiply_mult

	sbiw	Y,1
	
	dec	multiply_inner_counter
	brne	multiply_inner_loop
	
;..............................................................

	pop	ZH
	pop	ZL
	pop	YH
	pop	YL
	sbiw	X,1
	sbiw	Z,1
	dec	multiply_counter
	brne	multiply_outer_loop
	
;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,

	adiw	Y,1
	clr	multiply_data
	sub	YL,multiply_digits
	sbc	YH,multiply_data
	;;			Y points to MSB of result for return
	
;	clr	multiply_data
	sub	ZL,multiply_digits
	sbc	ZH,multiply_data
	adiw	Z,1
	;;			Z points to MSB of result
	
	mov	multiply_counter,multiply_digits
	lsl	multiply_counter ; *2

multiply_copy_result:
	ld	multiply_data,Z+
	st	Y+,multiply_data
	dec	multiply_counter
	brne	multiply_copy_result
	
	mov	multiply_counter,multiply_digits
	lsl	multiply_counter ; *2
	
multiply_free_result:	
	pop	multiply_data
	dec	multiply_counter
	brne	multiply_free_result
		
	pop	multiply_mult
	pop	multiply_data
	pop	multiply_inner_counter
	pop	multiply_counter
	pop	multiply_digits
	pop	multiply_h
	pop	multiply_l
	pop	XH
	pop	XL
	pop	YH
	pop	YL
	pop	ZH
	pop	ZL
	
	ret
	
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
