;***************************************************************************
; n_bit_fifo.asm
;***************************************************************************

; 18. March 2003
; Burkhard John
; Harlessstr. 9
; 40239 Duesseldorf
; email: burkhard-john@online-club.de

; buffer functions:

; bit_fifo_init
; bit_fifo_get_entry
; bit_fifo_write_entry
; bit_fifo_empty
; bit_fifo_full

;buffer--------------------------------------------------------------------
                ; memory usage
; in SRAM
;buffer:
;        .byte   length_of_buffer

; registers
;
; .def	data	=	r24     ; must be defined in main program, 
;				  r16-r25
;				  different form following registers
;				  registers must not be used by interrupts
;				  without push/pop

.def	bit_fifo_inc_b	=	r19	; r0-r25
.def	bit_fifo_inc_l	=	r20	; r0-r25
.def	bit_fifo_inc_h	=	r21	; r0-r25
.def	bit_fifo_XB	=	r22	; r0-r25
.def	bit_fifo_YB	=	r23	; r0-r25
	
; usable space is length_of_buffer - 12, so minimum length of buffer is 14
; for 2 bytes buffer. Maximum length is limited
; only by size of SRAM.

.equ    bit_fifo_length_l            =    0
.equ    bit_fifo_length_h            =    1
.equ    bit_fifo_in_pointer_b        =    2
.equ    bit_fifo_in_pointer_l        =    3
.equ    bit_fifo_in_pointer_h        =    4
.equ    bit_fifo_out_pointer_b       =    5
.equ    bit_fifo_out_pointer_l       =    6
.equ    bit_fifo_out_pointer_h       =    7
.equ	bit_fifo_bits		     =    8
.equ	bit_fifo_increment_b         =    9
.equ	bit_fifo_increment_l         =    10
.equ	bit_fifo_increment_h         =    11

.equ    length_of_bit_fifo_descriptor     =       12


;buffer--------------------------------------------------------------------
                                ; initializes a byte-buffer
                                ; pre:	
                                ; base address in ZH:ZL
                                ; length of buffer in YH:YL
                                ; number of bits in data
                                ; number must be a power of 2
                                ; post:	
                                ; bit fifo is ready,
                                ; no register modified
	
bit_fifo_init:
	push	YL
	push	YH
	
	push	bit_fifo_inc_b
	push	bit_fifo_inc_l
	push	bit_fifo_inc_h
	push	bit_fifo_YB
	
	std	Z+bit_fifo_bits,data

	mov	bit_fifo_inc_b,data
	clr	bit_fifo_inc_l
	clr	bit_fifo_inc_h
	ldi	bit_fifo_YB,5	
	clc

bit_fifo_init_loop:
	rol	bit_fifo_inc_b
	rol	bit_fifo_inc_l
	rol	bit_fifo_inc_h
	dec	bit_fifo_YB
	brne	bit_fifo_init_loop

	std	Z+bit_fifo_increment_b,bit_fifo_inc_b
	std	Z+bit_fifo_increment_l,bit_fifo_inc_l
	std	Z+bit_fifo_increment_h,bit_fifo_inc_h
	
	
        sbiw    YL,length_of_bit_fifo_descriptor
        std     Z+bit_fifo_length_l,YL
        std     Z+bit_fifo_length_h,YH
        mov     YL,ZL
        mov     YH,ZH
        adiw    YL,length_of_bit_fifo_descriptor
	
	clr	bit_fifo_YB
	std	Z+bit_fifo_out_pointer_b,bit_fifo_YB
        std     Z+bit_fifo_out_pointer_l,YL
        std     Z+bit_fifo_out_pointer_h,YH

        add	bit_fifo_YB,bit_fifo_inc_b
	adc	YL,bit_fifo_inc_l
	adc	YH,bit_fifo_inc_h
	
	std	Z+bit_fifo_in_pointer_b,bit_fifo_YB
        std     Z+bit_fifo_in_pointer_l,YL
        std     Z+bit_fifo_in_pointer_h,YH

	pop	bit_fifo_YB
	pop	bit_fifo_inc_h
	pop	bit_fifo_inc_l
	pop	bit_fifo_inc_b
	
	pop	YH
	pop	YL
        ret

;buffer--------------------------------------------------------------------
                        ; gets one byte from buffer in register data
                        ; basepointer of buffer in ZH:ZL
                        ; must not be called if buffer is empty, check first
bit_fifo_get_entry:
;        cli
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH
	push	bit_fifo_inc_b
	push	bit_fifo_inc_l
	push	bit_fifo_inc_h
                                        ; calculate new out pointer
        mov     XL,ZL
        mov     XH,ZH
        adiw    XL,length_of_bit_fifo_descriptor
        ldd     YL,Z+bit_fifo_length_l
        ldd     YH,Z+bit_fifo_length_h
        add     XL,YL
        adc     XH,YH
                                        ; in X : end_of_buffer +1

	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

	ldd	bit_fifo_inc_b,Z+bit_fifo_increment_b
	ldd	bit_fifo_inc_l,Z+bit_fifo_increment_l
	ldd	bit_fifo_inc_h,Z+bit_fifo_increment_h
	
	add	bit_fifo_YB,bit_fifo_inc_b
	adc	YL,bit_fifo_inc_l
	adc	YH,bit_fifo_inc_h
                                        ; if YH:YL out of buffer
                                        ; then set YH:YL to begin of buffer
        sub     XL,YL
        sbc     XH,YH
        brne    bit_fifo_get_entry_get

        mov     YL,ZL
        mov     YH,ZH
        adiw    YL,length_of_bit_fifo_descriptor
	clr	bit_fifo_YB

bit_fifo_get_entry_get:
                                        ; save new out pointer
	std	Z+bit_fifo_out_pointer_b,bit_fifo_YB
        std     Z+bit_fifo_out_pointer_l,YL
        std     Z+bit_fifo_out_pointer_h,YH

	ldd	bit_fifo_inc_b,Z+bit_fifo_out_pointer_b
	rcall	bit_fifo_get_mask
					; mask in bit_fifo_inc_h
                                        ; get entry from buffer
        ld      data,Y
	and	data,bit_fifo_inc_h

bit_fifo_get_entry_loop:
	tst	bit_fifo_YB
	breq	bit_fifo_get_end

	subi	bit_fifo_YB,32
	lsr	data
	rjmp	bit_fifo_get_entry_loop
	
bit_fifo_get_end:
	pop	bit_fifo_inc_h
	pop	bit_fifo_inc_l
	pop	bit_fifo_inc_b
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
;        sei
        ret

;buffer--------------------------------------------------------------------
                                ; writes one byte in register data to buffer
                                ; base address in ZH:ZL
                                ; if buffer is full, entry is ignored
                                ; no other register is modified
bit_fifo_write_entry:
	push	data
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH
	push	bit_fifo_inc_b
	push	bit_fifo_inc_l
	push	bit_fifo_inc_h

	ldd	bit_fifo_XB,Z+bit_fifo_in_pointer_b
        ldd     XL,Z+bit_fifo_in_pointer_l
        ldd     XH,Z+bit_fifo_in_pointer_h
	
	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

	sub	bit_fifo_YB,bit_fifo_XB
        sbc     YL,XL
        sbc     YH,XH
                                ; if buffer full, in_pointer = out_pointer
                                ; then ignore character
        breq    bit_fifo_write_entry_end

	ldd	bit_fifo_inc_b,Z+bit_fifo_in_pointer_b
	rcall	bit_fifo_get_mask
		
	ld	bit_fifo_inc_b,X
				; mask in bit_fifo_inc_h
	com	bit_fifo_inc_h
	and	bit_fifo_inc_h,bit_fifo_inc_b
				; old data in bit_fifo_inc_h, new bits masked
	
	ldd	bit_fifo_inc_b,Z+bit_fifo_out_pointer_b

bit_fifo_write_entry_shift_data_loop:
	tst	bit_fifo_inc_b
	breq	bit_fifo_write_entry_save
		
	lsl	data
	subi	bit_fifo_inc_b,32
	rjmp	bit_fifo_write_entry_shift_data_loop

bit_fifo_write_entry_save:
	or	bit_fifo_inc_h,data
	
	st	X,bit_fifo_inc_h

	ldd	bit_fifo_inc_b,Z+bit_fifo_increment_b
	ldd	bit_fifo_inc_l,Z+bit_fifo_increment_l
	ldd	bit_fifo_inc_h,Z+bit_fifo_increment_h

	add	bit_fifo_XB,bit_fifo_inc_l
	adc	XL,bit_fifo_inc_l
	adc	XH,bit_fifo_inc_h
	
                                ; check for input pointer out of buffer
        ldd     YL,Z+bit_fifo_length_l
        ldd     YH,Z+bit_fifo_length_h
        adiw    YL,length_of_bit_fifo_descriptor
        add     YL,ZL
        adc     YH,ZH
        sub     YL,XL
        sbc     YH,XH
        brne    bit_fifo_write_pointer_ok
                        ; if pointer out of buffer, set it to begin of buffer
        mov     XL,ZL
        mov     XH,ZH
        adiw    XL,length_of_bit_fifo_descriptor
	clr	bit_fifo_XB

bit_fifo_write_pointer_ok:
                                ; save new input pointer
	std	Z+bit_fifo_in_pointer_b,bit_fifo_XB
        std     Z+bit_fifo_in_pointer_l,XL
        std     Z+bit_fifo_in_pointer_h,XH

bit_fifo_write_entry_end:
	pop	bit_fifo_inc_h
	pop	bit_fifo_inc_l
	pop	bit_fifo_inc_b
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
	pop	data
        ret

;buffer--------------------------------------------------------------------
                                ; if buffer is empty, register data is cleard
                                ; else register data is set, 
                                ; zeroflag is set accordingly
                                ; base pointer in ZH:ZL
bit_fifo_empty:
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH
	push	bit_fifo_inc_b
	push	bit_fifo_inc_l
	push	bit_fifo_inc_h

        ser     data

	ldd	bit_fifo_inc_b,Z+bit_fifo_increment_b
	ldd	bit_fifo_inc_l,Z+bit_fifo_increment_l
	ldd	bit_fifo_inc_h,Z+bit_fifo_increment_h

	ldd	bit_fifo_XB,Z+bit_fifo_in_pointer_b
        ldd     XL,Z+bit_fifo_in_pointer_l
        ldd     XH,Z+bit_fifo_in_pointer_h
	
	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

        add	bit_fifo_YB,bit_fifo_inc_b
	adc	YL,bit_fifo_inc_l
	adc	YH,bit_fifo_inc_h

	cp	bit_fifo_XB,bit_fifo_YB
        cpc     XL,YL
        cpc     XH,YH
                        ; buffer empty if (in_pointer - (out_pointer+1))=0
        breq    bit_fifo_is_empty
	
	sub	bit_fifo_YB,bit_fifo_XB
        sbc     YL,XL
        sbc     YH,XH

	clr	bit_fifo_XB
        ldd     XL,Z+bit_fifo_length_l
        ldd     XH,Z+bit_fifo_length_h
	
	sub	bit_fifo_XB,bit_fifo_YB
        sbc     XL,YL
        sbc     XH,YH
                                        ; or buffer empty if
                                        ; pointers at opposite ends of buffer
                                        ; out_pointer+1 - in_pointer
                                        ; -length_of_buffer = 0
        brne    bit_fifo_empty_end

bit_fifo_is_empty:
        clr     data

bit_fifo_empty_end:
	tst	data
	pop	bit_fifo_inc_h
	pop	bit_fifo_inc_l
	pop	bit_fifo_inc_b
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
        ret

;buffer--------------------------------------------------------------------
                                ; if buffer is full, register data is cleard
                                ; else register data is sett, 
                                ; zeroflag is set accordingly
                                ; base pointer in ZH:ZL
bit_fifo_full:
	push	bit_fifo_XB
        push    XL
        push    XH
	push	bit_fifo_YB
        push    YL
        push    YH

        ser     data

	ldd	bit_fifo_XB,Z+bit_fifo_in_pointer_b
        ldd     XL,Z+bit_fifo_in_pointer_l
        ldd     XH,Z+bit_fifo_in_pointer_h
	
	ldd	bit_fifo_YB,Z+bit_fifo_out_pointer_b
        ldd     YL,Z+bit_fifo_out_pointer_l
        ldd     YH,Z+bit_fifo_out_pointer_h

	sub	bit_fifo_XB,bit_fifo_YB
        sbc     XL,YL
        sbc     XH,YH
                                ; buffer is full if in_pointer = out_pointer
        brne    bit_fifo_full_end

bit_fifo_is_full:
        clr     data

bit_fifo_full_end:
	tst	data
        pop     YH
        pop     YL
	pop	bit_fifo_YB
        pop     XH
        pop     XL
	pop	bit_fifo_XB
        ret
	
;buffer--------------------------------------------------------------------
; internal use only

bit_fifo_get_mask:
	
	ldd	bit_fifo_inc_l,Z+bit_fifo_bits
	clr	bit_fifo_inc_h

bit_fifo_get_mask_loop:
	sec
	rol	bit_fifo_inc_h         ; mask for bits
	dec	bit_fifo_inc_l
	brne	bit_fifo_get_mask_loop

	
bit_fifo_get_mask_shift_mask_loop:
	tst	bit_fifo_inc_b
	breq	bit_fifo_get_mask_shifted
	
	lsl	bit_fifo_inc_h
	subi	bit_fifo_inc_b,32
	rjmp	bit_fifo_get_mask_shift_mask_loop
	
bit_fifo_get_mask_shifted:
				; mask in bit_fifo_inc_h
	ret
	
;*****************************************************************************
