;**********************************************************************
;
;                             seq_keyer
;
;	  Electronic morse keyer with integrated PTT-sequencer
;
;                  (C)   Burkhard John, DK5JG
;                           22. Apr. 2004
;
;***********************************************************************

;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.

;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.

;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;*************************************************************

;*************************************************************
;
;
;       delay finder relay      off > on  : 4ms
;                               on  > off : 3ms
;       delay preamplifier      off > on  : 20ms
;                               on  > off : 140ms
;
;
;*************************************************************
.include		"m8def.inc"

.equ	EEPEND	=	$1FF	; missing in m8def.inc

;-------------------------------------------------------------

                                        ; system declaration

.equ    xtal_frequency		=       8000000 ; Hz  internal RC oscillator

						; set fuses to 00100100
						; Brownout enable, 4V
						; Startup 32K, 65ms
						; 8MHz internal oscillator
	
.equ	initial_delay		=	15	; 15ms

				; for memory repeat, press memory-button
	 			; twice in repeat_time ms
.equ	initial_double_time	=	200	; ms
.equ	max_double_time		=	1000	; ms
.equ	min_double_time		=	50	; ms
	
.equ	debouncing_time		=	50	; ms

.equ	initial_repeat_time	=	5	; s
.equ	max_repeat_time		=	65	; s

.equ	initial_qrs_time	=	1000	; ms
.equ	max_qrs_time		=	2500000 ; ms

.equ	initial_tune_time	=	5	; s
.equ	max_tune_time		=	60	; s 
.equ	tune_stop_time		=	100	; ms

.equ	max_test_time		=	60000	; ms

.equ	max_up_down_time	=	60000	; ms

.equ	length_of_cmd_memory		=	24
.equ	length_of_eeprom_memory_3	=	50
.equ	length_of_eeprom_memory_1 = ((EEPEND+1-eeprom_memory_1_start)/2)-1
.equ	length_of_eeprom_memory_2	=	length_of_eeprom_memory_1
.equ	length_of_command_buffer	=	13

						; fifo for external key
.equ	length_of_fifo			=	830
.equ	initial_fifo_bit_mask		=	1
	
						; QRS hold time
.equ	max_hold_time			=	60000
.equ	min_hold_time			=	500
.equ	initial_hold_time		=	2000
	
.equ	initial_gap_time		=	10 ; % of dot time
	
.equ	initial_profile			=	0
.equ	number_of_profiles		=	4
.equ	number_of_ptt_channels		=	6

				; time after last element from keyer before
				; shutdown ptt
.equ	initial_keyer_time		=	10

.equ	initial_element_length		=	150 ; for fifo


;-------------------------------------------------------------------

			; timer 1
			;
						; special functions off
.equ	timer_1_tccr1a_init	=	0
						; CK
.equ	timer_1_tccr1b_init	= 	1

.equ	t_1_overhead		=	15
.equ	timer_1_tic		=	8000	; Hz
.equ	sequencer_divider_val	=	8
.equ	tone_divider_val	=	5	; 1600 Hz for 800 Hz tone
.equ	tone_divider_dfcw	=	6
.equ	timer_1_reload		= t_1_overhead-(xtal_frequency/timer_1_tic)

;-------------------------------------------------------------------

					; enable timer 1 interrupt
.equ    timsk_init      	=       (1<<TOIE1) 

.equ	sequencer_timer_tic	=	timer_1_tic/sequencer_divider_val ; 1ms
	
.equ    one_second		=	1000 ; number of sequencer timer ticks for one second

;------------------------------------------------------------
	
				; Analog comparator
.equ    analog_comparator_init  =       (1<<ACD)	; off

;------------------------------------------------------------
	
				; ADC
                                ; Enable ADC, start conversion,
				; single mode, Interrupt disable,
                                ; Prescaler=128
.equ    adcsr_init		=	(1<<ADEN)|(1<<ADSC)|7
				; channel 0, AREF = 2.56 with capacitor
.equ	admux_init		=	(1<<REFS0)|(1<<REFS1)|0



;------------------------------------------------------------
                                        ; port B
.equ    memory_in_port          =       PINB
.equ	ptt_off_in_port		=	PINB
.equ	tune_input_port		=	PINB
.equ	ptt_5_outport		=	PORTB
.equ	ptt_6_outport		=	PORTB
.equ	dfcw_dash_port		=	PORTB

.equ	ptt_off_input		=	PB0
.equ	tune_input		=	PB1
.equ	memory_1_input		=	PB2
.equ	memory_2_input		=	PB3
.equ	memory_3_input		=	PB4
.equ	memory_4_input		=	PB5
.equ	ptt_5_out		=	PB6
.equ	ptt_6_out		=	PB7
.equ	dfcw_dash_out		=	PB7

					; all input but PB6, PB7
.equ	port_b_direction	=	(1<<ptt_5_out)|(1<<ptt_6_out)
					; all pullups on, outputs 0
.equ	port_b_init		=	~((1<<ptt_5_out)|(1<<ptt_6_out))

;------------------------------------------------------------

					; port C

.equ	on_off_switch_port_a	=	PINC
.equ	on_off_switch_port_b	=	PINC
.equ	dot_in_port		=	PINC
.equ	dash_in_port		=	PINC
.equ	ptt_in_a_port		=	PINC

.equ	on_off_switch_a		=	PC1
.equ	on_off_switch_b		=	PC2
.equ	ptt_in_a		=	PC3
.equ    dot_input		=       PC4
.equ    dash_input		=       PC5

					; all input
.equ	port_c_direction	=	0
					; pullup on on PC1-PC5
.equ	port_c_init		=	$3E

;------------------------------------------------------------

					; Port D
.equ	ptt_1_outport		=	PORTD
.equ	ptt_2_outport		=	PORTD
.equ	ptt_3_outport		=	PORTD
.equ	ptt_4_outport		=	PORTD
.equ	ptt_outport_direction	=	DDRD
.equ	tone_outport		=	PORTD
.equ	tone_inport		=	PIND
.equ	key_outport		=	PORTD
.equ	ptt_in_b_port		=	PIND
.equ	extern_in_port		=	PIND

					; input
.equ	ptt_in_b		=	PD2
.equ	extern_in		=	PD3

					; output
.equ	key_out			=	PD0
.equ	tone_out		=	PD1
.equ	ptt_4_out		=	PD4
.equ	ptt_3_out		=	PD5
.equ	ptt_2_out		=	PD6
.equ	ptt_1_out		=	PD7

					; ptt-lines, key and tone output, ptt_in, extern_in input
.equ	port_d_direction	=	(1<<key_out)|(1<<tone_out)|(1<<ptt_1_out)|(1<<ptt_2_out)|(1<<ptt_3_out)|(1<<ptt_4_out)
					; pullup on for inputs
.equ	port_d_init		=	(1<<ptt_in_a)|(1<<extern_in)

;------------------------------------------------------------
	
                                        ; bits in register flags
.equ    dash_sent       =       0       ; if 1, dash has been sent
.equ    dot_sent        =       1       ; if 1, dot has been sent
.equ    pause_sent      =       2       ; if 1, pause has been sent
.equ    part_of_char    =       3       ; if 1, a part of a character is sent
.equ    dot_mem_bit     =       4
.equ    dash_mem_bit    =       5
.equ    pause_mem_bit   =       6
.equ	memory_stop	=	7

.equ	flags_init	=	0
	
;------------------------------------------------------------
	
                                        ; bits in register memory_flags
.equ    memory_write    =       0       ; if 1, write mode
.equ    repeat_mode     =       1
.equ	command_memory	=	2
.equ	analyse_command	=	3
.equ	start_confirm	=	4
.equ	leading_zero	=	5
.equ	fifo_tic_rx	=	6
.equ	fifo_tic_tx	=	7
	
.equ	memory_flags_init	=	0
	
;------------------------------------------------------------
	
					; bits in register sequencer_flags
.equ	start_sequencer		=	0
.equ	start_sequencer_ext	=	1
.equ	sequencer_ok		=	2 
.equ	tx_key_signal		=	3
.equ	keyer_signal		=	4
.equ	extern_input_stop	=	5
.equ	timer_tic		=	6 ; timer tic 1ms
.equ	internal_buffer		=	7
	
.equ	sequencer_flags_init	=	0

;------------------------------------------------------------
	
					; bits in register more_flags
.equ	fifo_output_0		=	0
.equ	fifo_output_1		=	1
.equ	dfcw_fsk		=	2
.equ	dfcw_dash		=	3
.equ	measure_mode		=	4
.equ	memory_running		=	5

.equ	more_flags_init	=	0
	
;------------------------------------------------------------
	
					; bits in ptt_flags
.equ	ptt_off_all		=	0
.equ	ptt_on_a		=	1
.equ	ptt_on_b		=	2
.equ	ptt_reverse		=	3
.equ	ptt_pulse		=	4
	
;------------------------------------------------------------

					; bits in keyer_flags
.equ	keyer_dfcw		=	0 ; must be this bit
.equ	keyer_dfcw_invers	=	1 ; must be this bit
.equ	keyer_left_hand		=	2
.equ	keyer_dot_memory	=	3
.equ	keyer_dash_memory	=	4
.equ	keyer_buffered		=	5
.equ	keyer_qrs		=	6

.equ	keyer_flags_init	=	(1<<keyer_dot_memory)|(1<<keyer_dash_memory)
	
;------------------------------------------------------------

					; bits in register input
.equ	input_ptt_off		=	0
.equ	input_extern		=	1
.equ	input_ptt_in_a		=	2
.equ	input_ptt_in_b		=	3
.equ	input_on_off_a		=	4
.equ	input_on_off_b		=	5
.equ	input_tune		=	6
.equ	input_manual		=	7	

;------------------------------------------------------------
	                                        ; system states sequencer
.equ    sequencer_idle		=	$00
.equ	sequencer_ready		=	$01
	
.equ	start_ptt_1		=	$10
.equ	start_ptt_2		=	$11
.equ	start_ptt_3		=	$12
.equ	start_ptt_4		=	$13
.equ	start_ptt_5		=	$14
.equ	start_ptt_6		=	$15
	
.equ	run_ptt_1_up		=	$20
.equ	run_ptt_2_up		=	$21
.equ	run_ptt_3_up		=	$22
.equ	run_ptt_4_up		=	$23
.equ	run_ptt_5_up		=	$24
.equ	run_ptt_6_up		=	$25
	
.equ	stop_ptt_1		=	$30
.equ	stop_ptt_2		=	$31
.equ	stop_ptt_3		=	$32
.equ	stop_ptt_4		=	$33
.equ	stop_ptt_5		=	$34
.equ	stop_ptt_6		=	$35
	
.equ	run_ptt_1_down		=	$40
.equ	run_ptt_2_down		=	$41
.equ	run_ptt_3_down		=	$42
.equ	run_ptt_4_down		=	$43
.equ	run_ptt_5_down		=	$44
.equ	run_ptt_6_down		=	$45

.equ	stop_last_ptt		=	stop_ptt_6
	
;------------------------------------------------------------

					; system states keyer
.equ	keyer_idle			=	0
.equ	keyer_start_dot			=	1
.equ    keyer_send_dot			=	2
.equ	keyer_start_dash		=	3
.equ    keyer_send_dash			=       4	
.equ    keyer_start_pause_after_dot	=       5  
.equ    keyer_start_pause_after_dash	=       6	
.equ    keyer_send_pause_after_dot	=       7  
.equ    keyer_send_pause_after_dash	=       8
.equ	keyer_start_memory_pause	=	9
.equ	keyer_send_memory_pause		=	10
.equ    keyer_start_write_pause		=       11
.equ    keyer_send_write_pause		=       12
.equ	keyer_start_ptt_shutdown	=	13
.equ	keyer_wait_for_ptt_shutdown	=	14
.equ	keyer_wait_for_ptt		=	15
.equ	keyer_choose_pause		=	16

;------------------------------------------------------------
	
					; system states tune
.equ	tune_idle		=	0
.equ	tune_start		=	1
.equ	tune_running		=	2
.equ	tune_stop		=	3
.equ	tune_stop_wait		=	4

;------------------------------------------------------------

					; system states memory
.equ	memory_idle		=	0
.equ	memory_wait_for_release	=	1
.equ	memory_read_start	=	2
.equ	memory_start_double	=	3
.equ	memory_run_double	=	4
.equ	memory_read_run		=	5
.equ	memory_repeat_wait	=	7
.equ	memory_write_start	=	8
.equ	memory_write_run	=	9
.equ	memory_write_stop	=	10
.equ	memory_debounce_start	=	11
.equ	memory_debounce_run	=	12
.equ	memory_repeat_start	=	13
.equ	memory_repeat_run	=	14
.equ	memory_repeat_start_send =	15
.equ	memory_repeat_wait_for_release = 16
.equ	memory_write_wait_for_release = 17
.equ	memory_delay_send_start	=	18
.equ	memory_delay_send_run	=	19
.equ	memory_start_double_debounce =	20
.equ	memory_run_double_debounce =	21
.equ	memory_double_wait_for_release = 22
		
;------------------------------------------------------------

					; system states fifo input
.equ	fifo_input_idle		=	0
.equ	fifo_input_running	=	1
	
;------------------------------------------------------------
	
					; system states fifo output
.equ	fifo_output_idle	=	0
.equ	fifo_output_start_1	=	1
.equ	fifo_output_start_0	=	2
.equ	fifo_output_running_1	=	3
.equ	fifo_output_running_0	=	4
.equ	fifo_output_stop	=	5

;------------------------------------------------------------
                                        ; registers
.def	multiply_l		=	r0
.def	multiply_h		=	r1
.def    mem_bit_counter		=	r2
.def    mem_data		=	r3
.def	memory_keys		=	r4	
.def    input			=	r5		
.def	sequencer_divider	=	r6
.def	tone_divider		=	r7
.def	timer_l			=	r8
.def	timer_h			=	r9
.def    timer_x			=	r10
.def	sequencer_timer_l	=	r11
.def	sequencer_timer_h	=	r12
.def    soft_timer_l		=	r13
.def    soft_timer_h		=	r14
.def	soft_timer_x		=	r15
.def	sequencer_flags		=	r16
.def    memory_flags		=	r17
.def    flags			=	r18
.def	more_flags		=	r19
.def    counter			=	r20
.def	ZX			=	r21
.def	XX			=	r22
.def	data_x			=	r23 
.def    data			=	r24
.def    data_h			=	r25
;					XL
;					XH
;					YL
;					YH
;					ZL
;					ZH

;------------------------------------------------------------------------
;               some usefull macros
;                       if the original branch operation jumps too short
;                       usage:
;                       _brne destination
.macro _brne
        breq    PC+2
        rjmp    @0
.endmacro

.macro _breq
        brne    PC+2
        rjmp    @0
.endmacro
	
.macro _brcc
        brcs    PC+2
        rjmp    @0
.endmacro

.macro _brcs
        brcc    PC+2
        rjmp    @0
.endmacro
	
.macro _brsh
        brlo    PC+2
        rjmp    @0
.endmacro

.macro _brlo
        brsh    PC+2
        rjmp    @0
.endmacro
	
;****************************************************

                                ;EEPROM
.eseg
do_not_use:
	.db	$FF

eep_active_profile:
	.db	initial_profile

eep_0_profile:
eep_0_ptt_1_data:
eep_0_ptt_1_start_time_l:
	.db	LOW(initial_delay)
.equ	ptt_start_time_l	=	eep_0_ptt_1_start_time_l - eep_0_ptt_1_data
eep_0_ptt_1_start_time_h:
	.db	HIGH(initial_delay)
.equ	ptt_start_time_h	=	eep_0_ptt_1_start_time_h - eep_0_ptt_1_data
eep_0_ptt_1_stop_time_l:
	.db	LOW(initial_delay)
.equ	ptt_stop_time_l		=	eep_0_ptt_1_stop_time_l - eep_0_ptt_1_data
eep_0_ptt_1_stop_time_h:
	.db	HIGH(initial_delay)
.equ	ptt_stop_time_h		=	eep_0_ptt_1_stop_time_h - eep_0_ptt_1_data

eep_0_ptt_1_flags:
	.db	0
.equ	ptt_flags		=	eep_0_ptt_1_flags - eep_0_ptt_1_data

eep_0_ptt_2_data:
.equ	length_of_profile_data	=	eep_0_ptt_2_data - eep_0_ptt_1_data

eep_0_ptt_2_start_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_2_start_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_2_stop_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_2_stop_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_2_flags:
	.db	0

eep_0_ptt_3_data:
eep_0_ptt_3_start_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_3_start_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_3_stop_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_3_stop_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_3_flags:
	.db	0

eep_0_ptt_4_data:
eep_0_ptt_4_start_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_4_start_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_4_stop_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_4_stop_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_4_flags:
	.db	0


eep_0_ptt_5_data:
eep_0_ptt_5_start_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_5_start_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_5_stop_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_5_stop_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_5_flags:
	.db	(1<<ptt_off_all)


eep_0_ptt_6_data:
eep_0_ptt_6_start_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_6_start_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_6_stop_time_l:
	.db	LOW(initial_delay)
eep_0_ptt_6_stop_time_h:
	.db	HIGH(initial_delay)
eep_0_ptt_6_flags:
	.db	(1<<ptt_off_all)


eep_0_keyer_flags:
	.db	keyer_flags_init
.equ	keyer_flags	=	eep_0_keyer_flags - eep_0_profile
eep_0_keyer_time:
	.db	initial_keyer_time
.equ	keyer_time	=	eep_0_keyer_time - eep_0_profile
eep_0_tune_time:
	.db	initial_tune_time
.equ	tune_time	=	eep_0_tune_time - eep_0_profile
eep_0_repeat_time:
	.db	(initial_repeat_time)
.equ	repeat_time	=	eep_0_repeat_time - eep_0_profile
eep_0_repeat_double_time_l:
	.db	LOW(initial_double_time)
.equ	repeat_double_time_l	=	eep_0_repeat_double_time_l - eep_0_profile
eep_0_repeat_double_time_h:
	.db	HIGH(initial_double_time)
.equ	repeat_double_time_h	=	eep_0_repeat_double_time_h - eep_0_profile
eep_0_qrs_time_l:
	.db	LOW(initial_qrs_time)
.equ	qrs_time_l		=	eep_0_qrs_time_l - eep_0_profile
eep_0_qrs_time_h:
	.db	HIGH(initial_qrs_time)
.equ	qrs_time_h	=	eep_0_qrs_time_h - eep_0_profile
eep_0_qrs_time_x:
	.db	BYTE3(initial_qrs_time)
.equ	qrs_time_x	=	eep_0_qrs_time_x - eep_0_profile
eep_0_hold_time_l:
	.db	LOW(initial_hold_time)
.equ	hold_time_l		=	eep_0_hold_time_l - eep_0_profile
eep_0_hold_time_h:
	.db	HIGH(initial_hold_time)
.equ	hold_time_h	=	eep_0_hold_time_h - eep_0_profile
eep_0_hold_time_x:
	.db	BYTE3(initial_hold_time)
.equ	hold_time_x	=	eep_0_hold_time_x - eep_0_profile
eep_0_dfcw_gap_time:
	.db	initial_gap_time
.equ	dfcw_gap_time	=	eep_0_dfcw_gap_time - eep_0_profile


;.....................................................
eep_1_profile:
.equ	length_of_profile	=	eep_1_profile - eep_0_profile
	

eep_1_ptt_1_data:
eep_1_ptt_1_start_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_1_start_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_1_stop_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_1_stop_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_1_flags:
	.db	0


eep_1_ptt_2_data:
eep_1_ptt_2_start_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_2_start_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_2_stop_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_2_stop_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_2_flags:
	.db	0

eep_1_ptt_3_data:
eep_1_ptt_3_start_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_3_start_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_3_stop_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_3_stop_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_3_flags:
	.db	0

eep_1_ptt_4_data:
eep_1_ptt_4_start_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_4_start_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_4_stop_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_4_stop_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_4_flags:
	.db	0


eep_1_ptt_5_data:
eep_1_ptt_5_start_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_5_start_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_5_stop_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_5_stop_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_5_flags:
	.db	(1<<ptt_off_all)


eep_1_ptt_6_data:
eep_1_ptt_6_start_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_6_start_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_6_stop_time_l:
	.db	LOW(initial_delay)
eep_1_ptt_6_stop_time_h:
	.db	HIGH(initial_delay)
eep_1_ptt_6_flags:
	.db	(1<<ptt_off_all)

eep_1_keyer_flags:
	.db	keyer_flags_init
eep_1_keyer_time:
	.db	initial_keyer_time
eep_1_tune_time:
	.db	initial_tune_time
eep_1_repeat_time:
	.db	(initial_repeat_time)
eep_1_repeat_double_time_l:
	.db	LOW(initial_double_time)
eep_1_repeat_double_time_h:
	.db	HIGH(initial_double_time)
eep_1_qrs_time_l:
	.db	LOW(initial_qrs_time)
eep_1_qrs_time_h:
	.db	HIGH(initial_qrs_time)
eep_1_qrs_time_x:
	.db	BYTE3(initial_qrs_time)
eep_1_fifo_time_l:
	.db	LOW(initial_hold_time)
eep_1_fifo_time_h:
	.db	HIGH(initial_hold_time)
eep_1_fifo_time_x:
	.db	BYTE3(initial_hold_time)
eep_1_dfcw_gap_time:
	.db	initial_gap_time

;.....................................................

eep_2_profile:
eep_2_ptt_1_data:
eep_2_ptt_1_start_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_1_start_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_1_stop_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_1_stop_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_1_flags:
	.db	0


eep_2_ptt_2_data:
eep_2_ptt_2_start_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_2_start_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_2_stop_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_2_stop_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_2_flags:
	.db	0

eep_2_ptt_3_data:
eep_2_ptt_3_start_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_3_start_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_3_stop_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_3_stop_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_3_flags:
	.db	0

eep_2_ptt_4_data:
eep_2_ptt_4_start_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_4_start_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_4_stop_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_4_stop_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_4_flags:
	.db	0


eep_2_ptt_5_data:
eep_2_ptt_5_start_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_5_start_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_5_stop_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_5_stop_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_5_flags:
	.db	(1<<ptt_off_all)


eep_2_ptt_6_data:
eep_2_ptt_6_start_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_6_start_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_6_stop_time_l:
	.db	LOW(initial_delay)
eep_2_ptt_6_stop_time_h:
	.db	HIGH(initial_delay)
eep_2_ptt_6_flags:
	.db	(1<<ptt_off_all)

eep_2_keyer_flags:
	.db	keyer_flags_init
eep_2_keyer_time:
	.db	initial_keyer_time
eep_2_tune_time:
	.db	initial_tune_time
eep_2_repeat_time:
	.db	(initial_repeat_time)
eep_2_repeat_double_time_l:
	.db	LOW(initial_double_time)
eep_2_repeat_double_time_h:
	.db	HIGH(initial_double_time)
eep_2_qrs_time_l:
	.db	LOW(initial_qrs_time)
eep_2_qrs_time_h:
	.db	HIGH(initial_qrs_time)
eep_2_qrs_time_x:
	.db	BYTE3(initial_qrs_time)
eep_2_fifo_time_l:
	.db	LOW(initial_hold_time)
eep_2_fifo_time_h:
	.db	HIGH(initial_hold_time)
eep_2_fifo_time_x:
	.db	BYTE3(initial_hold_time)	
eep_2_dfcw_gap_time:
	.db	initial_gap_time

;.....................................................

eep_3_profile:

eep_3_ptt_1_data:
eep_3_ptt_1_start_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_1_start_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_1_stop_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_1_stop_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_1_flags:
	.db	0


eep_3_ptt_2_data:
eep_3_ptt_2_start_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_2_start_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_2_stop_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_2_stop_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_2_flags:
	.db	0

eep_3_ptt_3_data:
eep_3_ptt_3_start_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_3_start_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_3_stop_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_3_stop_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_3_flags:
	.db	0


eep_3_ptt_4_data:
eep_3_ptt_4_start_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_4_start_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_4_stop_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_4_stop_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_4_flags:
	.db	0


eep_3_ptt_5_data:
eep_3_ptt_5_start_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_5_start_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_5_stop_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_5_stop_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_5_flags:
	.db	(1<<ptt_off_all)


eep_3_ptt_6_data:
eep_3_ptt_6_start_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_6_start_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_6_stop_time_l:
	.db	LOW(initial_delay)
eep_3_ptt_6_stop_time_h:
	.db	HIGH(initial_delay)
eep_3_ptt_6_flags:
	.db	(1<<ptt_off_all)

eep_3_keyer_flags:
	.db	keyer_flags_init
eep_3_keyer_time:
	.db	initial_keyer_time
eep_3_tune_time:
	.db	initial_tune_time
eep_3_repeat_time:
	.db	(initial_repeat_time)
eep_3_repeat_delay_time_l:
	.db	LOW(initial_double_time)
eep_3_repeat_delay_time_h:
	.db	HIGH(initial_double_time)
eep_3_qrs_time_l:
	.db	LOW(initial_qrs_time)
eep_3_qrs_time_h:
	.db	HIGH(initial_qrs_time)
eep_3_qrs_time_x:
	.db	BYTE3(initial_qrs_time)
eep_3_fifo_time_l:
	.db	LOW(initial_hold_time)
eep_3_fifo_time_h:
	.db	HIGH(initial_hold_time)
eep_3_fifo_time_x:
	.db	BYTE3(initial_hold_time)
eep_3_dfcw_gap_time:
	.db	initial_gap_time

;.....................................................


eeprom_memory_cmd_start:
	.db	length_of_cmd_memory
	.db	$FF
	.byte	length_of_cmd_memory - 1

eeprom_memory_3_start:
	.db	length_of_eeprom_memory_3
	.db	$FF 
	.byte	length_of_eeprom_memory_3-1

eeprom_memory_1_start:
	.db	length_of_eeprom_memory_1
                                ;dk5jg into EEPROM for test
        .db     0b10010100      ;-.._
        .db     0b00100110      ;_-.-
        .db     0b00000101      ;__..
        .db     0b01010100      ;..._
        .db     0b00011010      ;_.--
        .db     0b10000010      ;-__-
        .db     0b10011111      ;-.||
	.byte	length_of_eeprom_memory_1 - 7

eeprom_memory_2_start:
	.db	length_of_eeprom_memory_2
	.db	$FF
	.byte	length_of_eeprom_memory_2 - 1

;******************************************************************
.cseg
start:
        rjmp    reset
        rjmp    extern_int_0
        rjmp    extern_int_1
        rjmp    timer_2_comp
	rjmp	timer_2_ovf
	rjmp	timer_1_capt
	rjmp	timer_1_compa
	rjmp	timer_1_compb
	rjmp	timer_1_ovf
	rjmp    timer_0_ovf
	rjmp	spi_stc
	rjmp	usart_rxc
	rjmp	usart_udre
	rjmp	usart_txc
	rjmp	adc_int
	rjmp    eeprom_ready
	rjmp    ana_comp
	rjmp	twi_int
	rjmp	spm_ready

;******************************************************************

	.include	"seq_keyer_interrupt.asm"
	
;******************************************************************

reset:
	ldi	data,LOW(RAMEND)
	out	SPL,data
	ldi	data,HIGH(RAMEND)
	out	SPH,data

        ldi     data,analog_comparator_init
        out     ACSR,data

        ldi     data,port_b_direction
        out     DDRB,data
        ldi     data,port_b_init
        out     PORTB,data

        ldi     data,port_c_direction
        out     DDRC,data
        ldi     data,port_c_init
        out     PORTC,data

        ldi     data,port_d_direction
        out     DDRD,data
        ldi     data,port_d_init
        out     PORTD,data

	ldi	data,adcsr_init
	out	ADCSR,data
	ldi	data,admux_init
	out	ADMUX,data

	ldi	data,timer_1_tccr1a_init
	out	TCCR1A,data

	ldi	data,timer_1_tccr1b_init
	out	TCCR1B,data
	
        ldi     data,timsk_init
        out     TIMSK,data

;--------------------------------------------------------
                                ; initialize variables
	clr	data
	sts	test_time_on_l,data
	sts	test_time_on_h,data
	sts	test_time_off_l,data
	sts	test_time_off_h,data

	ldi	data,LOW(initial_element_length)
	ldi	data_h,HIGH(initial_element_length)
	ldi	data_x,BYTE3(initial_element_length)
	sts	element_length_l,data
	sts	element_length_h,data_h
	sts	element_length_x,data_x
	
	call	calculate_dfcw_gap_time

	ldi	data,initial_fifo_bit_mask
	sts	fifo_bits,data
	ldi	ZL,LOW(fifo)
	ldi	ZH,HIGH(fifo)
	ldi	YL,LOW(length_of_fifo)
	ldi	YH,HIGH(length_of_fifo)
	call	bit_fifo_init

	ldi	ZL,LOW(2*pgm_ptt_data)
	ldi	ZH,HIGH(2*pgm_ptt_data)
	
	ldi	YL,LOW(ram_ptt_data)
	ldi	YH,HIGH(ram_ptt_data)
	ldi	counter,number_of_ptt_channels * length_of_pgm_ptt_data

get_pgm_ptt_data_loop:
	lpm	data,Z
	st	Y+,data
	adiw	ZL,2
	dec	counter
	brne	get_pgm_ptt_data_loop
	
        ldi     flags,flags_init
        ldi     more_flags,more_flags_init
        ldi     memory_flags,memory_flags_init
	ldi	sequencer_flags,sequencer_flags_init

        ldi     data,keyer_idle
	sts	keyer_state,data
	ldi	data,sequencer_idle
	sts	sequencer_state,data
	ldi	data,memory_idle
	sts	memory_state,data
	ldi	data,tune_idle
	sts	tune_state,data
	ldi	data,fifo_input_idle
	sts	fifo_input_state,data
	ldi	data,fifo_output_idle
	sts	fifo_output_state,data
	ldi	data,tone_divider_val
	sts	tone,data

	call	get_profile
	call	profile_initialize
                                        ; enable all interrupts
        sei


;*************************************************************

main:
	rcall	read_input_lines
	rcall	get_speed
	rcall	sequencer
	rcall	memory
	rcall	command
	rcall	keyer
	rcall	tune
	rcall	external_key_read
	rcall	external_key_send
	rcall	key_transmitter
	call	measure			 
	rjmp	main
	
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

read_input_lines:	
	clr	data
 	sbis	ptt_off_in_port,ptt_off_input  
 	sbr	data,(1<<input_ptt_off)

	sbis	ptt_in_a_port,ptt_in_a
	sbr	data,(1<<input_ptt_in_a)

	sbis	ptt_in_b_port,ptt_in_b
	sbr	data,(1<<input_ptt_in_b)

	sbis	extern_in_port,extern_in
	sbr	data,(1<<input_extern)

	sbis	on_off_switch_port_a,on_off_switch_a
	sbr	data,(1<<input_on_off_a)

	sbis	on_off_switch_port_b,on_off_switch_b
	sbr	data,(1<<input_on_off_b)

	sbis	tune_input_port,tune_input
	sbr	data,(1<<input_tune)

	mov	input,data
		
read_keyer_input:
	clr	data_h
	cbr	flags,(1<<memory_stop)
	
	sbrc	input,input_tune
	sbr	flags,(1<<memory_stop)

	sbis	dot_in_port,dot_input
	sbr	data_h,(1<<dot_mem_bit)
	
	sbis	dash_in_port,dash_input
	sbr	data_h,(1<<dash_mem_bit)

	sbrc	data_h,dot_mem_bit
	sbr	flags,(1<<memory_stop)

	sbrc	data_h,dash_mem_bit
	sbr	flags,(1<<memory_stop)
	
	lds	data,ram_ptt_profile + keyer_flags
	
	sbrc	data,keyer_left_hand
	rjmp	read_keyer_input_left_hand

	or	flags,data_h
	rjmp	read_keyer_input_end
	
read_keyer_input_left_hand:	
	sbrc	data_h,dot_mem_bit
	sbr	flags,(1<<dash_mem_bit)
	
	sbrc	data_h,dash_mem_bit
	sbr	flags,(1<<dot_mem_bit)

read_keyer_input_end:
	
read_memory_keys:
	in	memory_keys,memory_in_port
	com	memory_keys
	ldi	data,(1<<memory_4_input)|(1<<memory_1_input)|(1<<memory_2_input)|(1<<memory_3_input)
	and	memory_keys,data
	breq	read_memory_input_lines_end
	
	sbr	flags,(1<<memory_stop)
	
read_memory_input_lines_end:
	
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

get_speed:
	sbis	ADCSR,ADIF
	rjmp	get_speed_do_it

	in	data,ADCL
	in	data_h,ADCH
	clc
	ror	data_h
	ror	data
	
	sts	pot_time_l,data
	sts	pot_time_h,data_h
	
	sbi	ADCSR,ADIF
	sbi	ADCSR,ADSC
	
get_speed_do_it:
	lds	data,pot_time_l
	lds	data_h,pot_time_h
	ldi	ZL,LOW(2*speed_table)
	ldi	ZH,HIGH(2*speed_table)
	add	ZL,data
	adc	ZH,data_h
	lpm	data,Z
	clr	data_h
	clr	data_x

	sbrs	more_flags,memory_running
	rjmp	get_speed_set
		
	lds	counter,ram_ptt_profile+keyer_flags
	andi	counter,(1<<keyer_qrs)|(1<<keyer_dfcw)|(1<<keyer_dfcw_invers)
	breq	get_speed_set

	sbrc	memory_flags,start_confirm
	rjmp	get_speed_set
	
	mov	counter,memory_keys
	andi	counter,(1<<memory_4_input)|(1<<memory_3_input)|(1<<memory_2_input)|(1<<memory_1_input)
	brne	get_speed_set
	
get_speed_qrs_dfcw:
	lds	data,ram_ptt_profile+qrs_time_l
	lds	data_h,ram_ptt_profile+qrs_time_h
	lds	data_x,ram_ptt_profile+qrs_time_x
	clc
	rol	data
	rol	data_h
	rol	data_x
	
get_speed_set:	
	sts	dot_time_l,data
	sts	dot_time_h,data_h
	sts	dot_time_x,data_x
	
	ret
	
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

key_transmitter:
	lds	data,ram_ptt_profile+keyer_flags
	andi	data,(1<<keyer_buffered)|(1<<keyer_qrs)|(1<<keyer_dfcw)|(1<<keyer_dfcw_invers)
	brne	key_transmitter_buffered
	
	mov	data,sequencer_flags
	andi	data,(1<<tx_key_signal)|(1<<keyer_signal)
	breq	key_transmitter_off
	sbi	key_outport,key_out
	rjmp	key_transmitter_end

key_transmitter_buffered:
	sbrs	sequencer_flags,keyer_signal
	cbr	sequencer_flags,(1<<internal_buffer)
	sbrc	sequencer_flags,keyer_signal
	sbr	sequencer_flags,(1<<internal_buffer)

	ldi	data,tone_divider_val
	lds	data_h,ram_ptt_profile+keyer_flags
	andi	data_h,(1<<keyer_dfcw)|(1<<keyer_dfcw_invers)
	breq	key_transmitter_normal_tone
	
	sbrc	more_flags,dfcw_fsk
	sbi	dfcw_dash_port,dfcw_dash_out
	sbrc	more_flags,dfcw_dash
	ldi	data,tone_divider_val
	
	sbrs	more_flags,dfcw_fsk
	cbi	dfcw_dash_port,dfcw_dash_out
	sbrs	more_flags,dfcw_dash
	ldi	data,tone_divider_dfcw
	
key_transmitter_normal_tone:
	sts	tone,data
	
	mov	data,sequencer_flags
	andi	data,(1<<tx_key_signal)
	breq	key_transmitter_off
	sbi	key_outport,key_out
	rjmp	key_transmitter_end

key_transmitter_off:
	cbi	key_outport,key_out
;	rjmp	key_transmitter_end	
	
key_transmitter_end:
	ret
	
	
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	.include	"seq_keyer_sequencer.asm"
	.include	"seq_keyer_tune.asm"
	.include	"seq_keyer_external_key.asm"
	.include	"seq_keyer_memory.asm"
	.include	"seq_keyer_command.asm"
	.include	"seq_keyer_keyer.asm"
	.include	"seq_keyer_measure.asm"
	.include	"bit_fifo_d.asm"
	.include	"multiply_n_n_to_2n.asm"
		

	
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++

;-------------------------------------------------------------------------
;                       write one byte to the eeprom
                        ; address in XH:XL, data in reg. data
eeprom_write_byte:
                                ; wait for end of previous write cycle
        sbic    EECR,EEWE
        rjmp    eeprom_write_byte

        cli
        out     EEARL,XL
        out     EEARH,XH
        out     EEDR,data
        sbi     EECR,EEMWE
        sbi     EECR,EEWE
        sei

        ret

;-------------------------------------------------------------------------
;			clear address register of eeprom
eeprom_clear_address:
                                ; wait for end of previous write cycle
        sbic    EECR,EEWE
        rjmp    eeprom_clear_address

        push    data
        clr     data
        out     EEARL,data
        out     EEARH,data
        pop     data

        ret
;---------------------------------------------------------------
                        ; read one byte from eeprom.
                        ; address in XH:XL, data in reg. data
eeprom_get_byte:
                                        ; wait for end of running write cycle
        sbic    EECR,EEWE
        rjmp    eeprom_get_byte

        out     EEARL,XL
        out     EEARH,XH

        sbi     EECR,EERE               ; start EEPROM-read
        in      data,EEDR

        push    data
        clr     data
        out     EEARL,data
        out     EEARH,data
        pop     data

        ret

; ......................................................

				; in data number of sequencer channel
				; result is base address of data set
				; profile in Y
				; ptt data in Z
get_profile_base_addresses:
	lds	data,sequencer_state

get_profile_base_addresses_a:	
	andi	data,$07
	mov	data_h,data
	ldi	YL,LOW(ram_ptt_profile)
	ldi	YH,HIGH(ram_ptt_profile)
	ldi	XL,LOW(length_of_profile_data)
	ldi	XH,HIGH(length_of_profile_data)
	
get_profile_base_address_loop_1:	
	tst	data
	breq	get_profile_base_address_next

	add	YL,XL
	adc	YH,XH
	dec	data
	rjmp	get_profile_base_address_loop_1

get_profile_base_address_next:	
	ldi	ZL,LOW(ram_ptt_data)
	ldi	ZH,HIGH(ram_ptt_data)
	ldi	XL,LOW(length_of_pgm_ptt_data)
	ldi	XH,HIGH(length_of_pgm_ptt_data)
	
get_profile_base_address_loop_2:	
	tst	data_h
	breq	get_profile_base_address_end

	add	ZL,XL
	adc	ZH,XH
	dec	data_h
	rjmp	get_profile_base_address_loop_2


get_profile_base_address_end:
	ret

; ......................................................
	
profile_initialize:
	ldi	counter,number_of_ptt_channels
	
profile_initialize_loop:
	dec	counter
	mov	data,counter
	rcall	get_profile_base_addresses_a
	
	ldd	XL,Z+ptt_port
	clr	XH
	
	
	ldd	data,Y+ptt_flags

	sbrc	data,ptt_off_all
	rjmp	profile_initialize_reverse

	sbrs	data,ptt_on_a
	rjmp	profile_initialize_off_b
	
	sbrc	input,input_on_off_a
	rjmp	profile_initialize_ptt_off

profile_initialize_off_b:
	sbrs	data,ptt_on_b	
	rjmp	profile_initialize_ptt_go_on
	
	sbrc	input,input_on_off_b
	rjmp	profile_initialize_ptt_off

profile_initialize_ptt_go_on:
	sbrs	data,ptt_reverse
	rjmp	profile_initialize_reverse
	
profile_initialize_normal:		
	ldd	data_h,Z+ptt_bit
	ld	data,X
	or	data,data_h
	st	X,data
	rjmp	profile_initialize_channel_done
	
profile_initialize_ptt_off:
	
profile_initialize_reverse:
	ldd	data_h,Z+ptt_bit
	com	data_h	
	ld	data,X
	and	data,data_h
	st	X,data
	
profile_initialize_channel_done:
	tst	counter
	brne	profile_initialize_loop

	ret
	
;..................................................................

get_profile:
	rcall	get_profile_address
	
	ldi	YL,LOW(ram_ptt_profile)
	ldi	YH,HIGH(ram_ptt_profile)
	ldi	counter,length_of_profile

get_profile_loop:
	rcall	eeprom_get_byte
	st	Y+,data
	adiw	XL,1
	dec	counter
	brne	get_profile_loop

	ret
	
;..................................................................
	
get_profile_address:				
	ldi	XL,LOW(eep_active_profile)
	ldi	XH,HIGH(eep_active_profile)
	rcall	eeprom_get_byte
	
	ldi	YL,LOW(length_of_profile)
	ldi	YH,HIGH(length_of_profile)

	mul	YL,data
	mov	ZL,multiply_l
	mov	ZH,multiply_h
	mul	YH,data
	add	ZH,multiply_h
	
	ldi	XL,LOW(eep_0_profile)
	ldi	XH,HIGH(eep_0_profile)
	
	add	XL,ZL
	adc	XH,ZH
	
	ret
	
;..................................................................
	
decrement_timer:		
	cbr	sequencer_flags,(1<<timer_tic)
	ldi	data,LOW(1)
	ldi	data_h,HIGH(1)
	ldi	data_x,BYTE3(1)
	sub	timer_l,data
	sbc	timer_h,data_h
	sbc	timer_x,data_x
	ret
	
		
;............................................................
	
calculate_dfcw_gap_time:
	push	XL
	push	XH
	push	YH
	push	ZL
	push	ZH	
	
	lds	data,ram_ptt_profile+qrs_time_l	
	lds	data_h,ram_ptt_profile+qrs_time_h
	lds	data_x,ram_ptt_profile+qrs_time_x
	clc
	rol	data
	rol	data_h
	rol	data_x
		
	push	data
	push	data_h
	push	data_x
	
	lds	data,ram_ptt_profile+dfcw_gap_time
	clr	data_h

	push	data
	push	data_h
	push	data_h
	
	ldi	data,3
	push	data
	rcall	multiply_n_n_2n
	pop	data
	pop	data
	pop	data
	pop	data
	pop	data_x
	pop	data_h
	pop	data
	
	ldi	counter,100
	ldi	XL,LOW(-1)
	ldi	XH,HIGH(-1)
	ldi	YH,BYTE3(-1)
	ldi	ZL,1
	clr	ZH
	
calculate_dfcw_gap_time_div_loop:
	add	XL,ZL
	adc	XH,ZH
	adc	YH,ZH
	sub	data,counter
	sbc	data_h,ZH
	sbc	data_x,ZH
	brcc	calculate_dfcw_gap_time_div_loop	

	sts	gap_time_dfcw_l,XL
	sts	gap_time_dfcw_h,XH
	sts	gap_time_dfcw_x,YH

	pop	ZH
	pop	ZL
	pop	YH
	pop	XH
	pop	XL
	ret
		
;****************************************************

	.include	"morse_code_table.asm"
	.include	"speed_table.asm"
	
;****************************************************

pgm_ptt_data:	
pgm_ptt_1:	
pgm_ptt_1_port:
	.db	ptt_1_outport + $20
.equ	ptt_port		=	pgm_ptt_1_port - pgm_ptt_1
pgm_ptt_1_bit:
	.db	(1<<ptt_1_out)
.equ	ptt_bit			=	pgm_ptt_1_bit - pgm_ptt_1
pgm_ptt_1_restart_state:
	.db	start_ptt_1
.equ	ptt_restart_state	=	pgm_ptt_1_restart_state - pgm_ptt_1
pgm_ptt_1_up_run_state:
	.db	run_ptt_1_up
.equ	ptt_up_run_state	=	pgm_ptt_1_up_run_state - pgm_ptt_1
pgm_ptt_1_up_next_state:
	.db	start_ptt_2
.equ	ptt_up_next_state	=	pgm_ptt_1_up_next_state - pgm_ptt_1
pgm_ptt_1_down_next_state:
	.db	sequencer_idle
.equ	ptt_down_next_state	=	pgm_ptt_1_down_next_state - pgm_ptt_1
pgm_ptt_1_down_run_state:
	.db	run_ptt_1_down
.equ	ptt_down_run_state	=	pgm_ptt_1_down_run_state - pgm_ptt_1
pgm_ptt_1_stop:
	.db	stop_ptt_1
.equ	ptt_stop		=	pgm_ptt_1_stop - pgm_ptt_1


pgm_ptt_2:
.equ	length_of_pgm_ptt_data	=	pgm_ptt_2 - pgm_ptt_1
pgm_ptt_2_port:
	.db	ptt_2_outport + $20
pgm_ptt_2_bit:
	.db	(1<<ptt_2_out)
pgm_ptt_2_restart_state:
	.db	start_ptt_2
pgm_ptt_2_up_run_state:
	.db	run_ptt_2_up
pgm_ptt_2_up_next_state:
	.db	start_ptt_3 
pgm_ptt_2_down_next_state:
	.db	stop_ptt_1
pgm_ptt_2_down_run_state:
	.db	run_ptt_2_down
pgm_ptt_2_stop:
	.db	stop_ptt_2

pgm_ptt_3:
pgm_ptt_3_port:
	.db	ptt_3_outport + $20
pgm_ptt_3_bit:
	.db	(1<<ptt_3_out)
pgm_ptt_3_restart_state:
	.db	start_ptt_3
pgm_ptt_3_up_run_state:
	.db	run_ptt_3_up
pgm_ptt_3_up_next_state:
	.db	start_ptt_4 ; 
pgm_ptt_3_down_next_state:
	.db	stop_ptt_2
pgm_ptt_3_down_run_state:
	.db	run_ptt_3_down
pgm_ptt_3_stop:
	.db	stop_ptt_3

pgm_ptt_4:
pgm_ptt_4_port:
	.db	ptt_4_outport + $20
pgm_ptt_4_bit:
	.db	(1<<ptt_4_out)
pgm_ptt_4_restart_state:
	.db	start_ptt_4
pgm_ptt_4_up_run_state:
	.db	run_ptt_4_up
pgm_ptt_4_up_next_state:
	.db	start_ptt_5
pgm_ptt_4_down_next_state:
	.db	stop_ptt_3
pgm_ptt_4_down_run_state:
	.db	run_ptt_4_down
pgm_ptt_4_stop:
	.db	stop_ptt_4	

pgm_ptt_5:
pgm_ptt_5_port:
	.db	ptt_5_outport + $20
pgm_ptt_5_bit:
	.db	(1<<ptt_5_out)
pgm_ptt_5_restart_state:
	.db	start_ptt_5
pgm_ptt_5_up_run_state:
	.db	run_ptt_5_up
pgm_ptt_5_up_next_state:
	.db	start_ptt_6
pgm_ptt_5_down_next_state:
	.db	stop_ptt_4
pgm_ptt_5_down_run_state:
	.db	run_ptt_5_down
pgm_ptt_5_stop:
	.db	stop_ptt_5

pgm_ptt_6:
pgm_ptt_6_port:
	.db	ptt_6_outport + $20
pgm_ptt_6_bit:
	.db	(1<<ptt_6_out)
pgm_ptt_6_restart_state:
	.db	start_ptt_6
pgm_ptt_6_up_run_state:
	.db	run_ptt_6_up
pgm_ptt_6_up_next_state:
	.db	sequencer_ready
pgm_ptt_6_down_next_state:
	.db	stop_ptt_5
pgm_ptt_6_down_run_state:
	.db	run_ptt_6_down
pgm_ptt_6_stop:
	.db	stop_ptt_6

;****************************************************************
;	RAM
.dseg
	
dot_time_l:	
	.byte	1
dot_time_h:	
	.byte	1
dot_time_x:
	.byte	1
	
gap_time_dfcw_l:	
	.byte	1
gap_time_dfcw_h:	
	.byte	1
gap_time_dfcw_x:
	.byte	1
	
pot_time_l:
	.byte	1
pot_time_h:
	.byte	1

memory_base_l:
	.byte	1
memory_base_h:
	.byte	1
memory_end_l:
	.byte	1
memory_end_h:
	.byte	1
memory_pointer_l:
	.byte	1
memory_pointer_h:
	.byte	1

keyer_state:
	.byte	1

memory_state:
	.byte	1

sequencer_state:
	.byte	1
	
tune_state:
	.byte	1
	
old_input:
	.byte	1

tone:
	.byte	1
	
test_time_l:
	.byte	1	
test_time_h:
	.byte	1
	
test_time_on_l:
	.byte	1	
test_time_on_h:
	.byte	1
	
test_time_off_l:
	.byte	1	
test_time_off_h:
	.byte	1
	
element_length_l:
	.byte	1
element_length_h:
	.byte	1
element_length_x:
	.byte	1
	
element_length_counter_l:
	.byte	1
element_length_counter_h:
	.byte	1
element_length_counter_x:
	.byte	1
	
ram_ptt_data:
	.byte	number_of_ptt_channels * length_of_pgm_ptt_data
	
ram_ptt_profile:	
	.byte	length_of_profile

fifo_input_state:
	.byte	1
fifo_output_state:
	.byte	1

fifo_bits:
	.byte	1
	
fifo:
	.byte	length_of_fifo
	
command_buffer:
	.byte	length_of_command_buffer

;****************************************************************
	
