
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

memory:
	lds	data,memory_state
	
	cpi	data,memory_idle
	breq	state_memory_idle

	cpi	data,memory_wait_for_release
	_breq	state_memory_wait_for_release

	cpi	data,memory_start_double
	_breq	state_memory_start_double

	cpi	data,memory_run_double
	_breq	state_memory_run_double

	cpi	data,memory_start_double_debounce
	_breq	state_memory_start_double_debounce

	cpi	data,memory_run_double_debounce
	_breq	state_memory_run_double_debounce

	cpi	data,memory_read_start
	_breq	state_memory_read_start

	cpi	data,memory_read_run
	_breq	state_memory_read_run

	cpi	data,memory_write_start
	_breq	state_memory_write_start

	cpi	data,memory_write_run
	_breq	state_memory_write_run

	cpi	data,memory_write_stop
	_breq	state_memory_write_stop

	cpi	data,memory_debounce_start
	_breq	state_memory_debounce_start

	cpi	data,memory_debounce_run
	_breq	state_memory_debounce_run

	cpi	data,memory_repeat_start
	_breq	state_memory_repeat_start

	cpi	data,memory_repeat_run
	_breq	state_memory_repeat_run

	cpi	data,memory_repeat_start_send
	_breq	state_memory_repeat_start_send

	cpi	data,memory_repeat_wait_for_release
	_breq	state_memory_repeat_wait_for_release

	cpi	data,memory_double_wait_for_release
	_breq	state_memory_double_wait_for_release

	cpi	data,memory_write_wait_for_release
	_breq	state_memory_write_wait_for_release

	cpi	data,memory_delay_send_start
	_breq	state_memory_delay_send_start

	cpi	data,memory_delay_send_run
	_breq	state_memory_delay_send_run
	
					; in case of error
	ldi	data,memory_idle
	rjmp	memory_set_new_state

;............................................................

state_memory_idle:	
	sbrc	memory_keys,memory_1_input
	rjmp	state_memory_idle_1

	sbrc	memory_keys,memory_2_input
	rjmp	state_memory_idle_2

	sbrc	memory_keys,memory_3_input
	rjmp	state_memory_idle_3

	sbrc	memory_keys,memory_4_input
	rjmp	state_memory_idle_4

	sbrs	memory_flags,start_confirm
	rjmp	memory_end
	
	ldi	data,memory_delay_send_start
	rjmp	memory_set_new_state
	
state_memory_idle_1:		
	cbr	memory_flags,(1<<command_memory)
	ldi	XL,LOW(eeprom_memory_1_start)
	ldi	XH,HIGH(eeprom_memory_1_start)
	rjmp	state_memory_get_memory_addresses
	
state_memory_idle_2:
	cbr	memory_flags,(1<<command_memory)		
	ldi	XL,LOW(eeprom_memory_2_start)
	ldi	XH,HIGH(eeprom_memory_2_start)
	rjmp	state_memory_get_memory_addresses
	
state_memory_idle_3:
	cbr	memory_flags,(1<<command_memory)		
	ldi	XL,LOW(eeprom_memory_3_start)
	ldi	XH,HIGH(eeprom_memory_3_start)
	rjmp	state_memory_get_memory_addresses
	
state_memory_idle_4:	
	sbr	memory_flags,(1<<command_memory)	
	ldi	XL,LOW(eeprom_memory_cmd_start)
	ldi	XH,HIGH(eeprom_memory_cmd_start)
	
state_memory_get_memory_addresses:
	rcall	memory_get_memory_addresses
	
	ldi	data,memory_debounce_start		
	rjmp	memory_set_new_state

;............................................................
	
state_memory_debounce_start:
	ldi	data,LOW(debouncing_time)
	ldi	data_h,HIGH(debouncing_time)
	ldi	data_x,BYTE3(debouncing_time)
	mov	timer_l,data
	mov	timer_h,data_h
	mov	timer_x,data_x

	ldi	data,memory_debounce_run
	rjmp	memory_set_new_state

;............................................................

state_memory_debounce_run:
	sbrs	sequencer_flags,timer_tic
	rjmp	memory_end

	rcall	decrement_timer 
	_brne	memory_end
	
	ldi	data,memory_wait_for_release
	rjmp	memory_set_new_state

;............................................................
	
state_memory_wait_for_release:
	tst	memory_keys
	breq	state_memory_keys_released
	
	mov	data,flags
	andi	data,(1<<dot_mem_bit)|(1<<dash_mem_bit)
	_breq	memory_end
	
	ldi	data,memory_write_start
	rjmp	memory_set_new_state
	
	
state_memory_keys_released:
	ldi	data,memory_start_double
	rjmp	memory_set_new_state

;............................................................
	
state_memory_start_double:
	lds	timer_l,ram_ptt_profile+repeat_double_time_l
	lds	timer_h,ram_ptt_profile+repeat_double_time_h
	clr	timer_x

	ldi	data,memory_run_double
	rjmp	memory_set_new_state

;............................................................
	
state_memory_run_double:
	sbrs	sequencer_flags,timer_tic
	rjmp	memory_end
	
	tst	memory_keys
	breq	state_memory_run_double_no_repeat
	
	sbr	memory_flags,(1<<repeat_mode)

state_memory_run_double_no_repeat:	
	rcall	decrement_timer 
	_brne	memory_end
	
	ldi	data,memory_read_start
	rjmp	memory_set_new_state

;............................................................
	
state_memory_start_double_debounce:
	lds	timer_l,ram_ptt_profile+repeat_double_time_l
	lds	timer_h,ram_ptt_profile+repeat_double_time_h
	clr	timer_x

	ldi	data,memory_run_double_debounce
	rjmp	memory_set_new_state

;............................................................
	
state_memory_run_double_debounce:
	sbrs	sequencer_flags,timer_tic
	rjmp	memory_end

	rcall	decrement_timer 
	_brne	memory_end
	
	ldi	data,memory_double_wait_for_release
	rjmp	memory_set_new_state

;............................................................
	
state_memory_double_wait_for_release:	
	tst	memory_keys
	_brne	memory_end
	
	sbr	memory_flags,(1<<repeat_mode)
	ldi	data,memory_read_start
	rjmp	memory_set_new_state
		
;............................................................
	
state_memory_read_start:
	tst	memory_keys
	_brne	memory_end
	
state_memory_repeat_start_send:
	sbr	more_flags,(1<<memory_running)
	lds	data,memory_base_l
	lds	data_h,memory_base_h
	sts	memory_pointer_l,data
	sts	memory_pointer_h,data_h
	clr	mem_bit_counter

	ldi	data,memory_read_run
	rjmp	memory_set_new_state

;............................................................
	
state_memory_read_run:
	sbrc	flags,memory_stop
	rjmp	state_memory_read_stop_all

        sbrc    flags,part_of_char
        rjmp    memory_end
	
        tst     mem_bit_counter
        brne    state_memory_read_run_send_byte

	ldi	data,4
	mov	mem_bit_counter,data
	
	lds	XL,memory_pointer_l
	lds	XH,memory_pointer_h
	adiw	X,1
	sts	memory_pointer_l,XL
	sts	memory_pointer_h,XH
	
	lds	YL,memory_end_l
	lds	YH,memory_end_h
	
	cp	XL,YL
	cpc	XH,YH
	breq	state_memory_read_stop

	rcall	eeprom_get_byte
	mov	mem_data,data
	
state_memory_read_run_send_byte:
	dec	mem_bit_counter
        mov     data,mem_data
                                        ; shift remaining bits up
        lsl     mem_data
        lsl     mem_data

        andi    data,0b11000000

        cpi     data,0b00000000
        breq    state_memory_send_pause

        cpi     data,0b01000000
        breq    state_memory_send_dot

        cpi     data,0b10000000
        breq	state_memory_send_dash

                                        ; else end of memory
	rjmp	state_memory_read_stop

state_memory_send_pause:
        sbr     flags,(1<<pause_mem_bit)|(1<<part_of_char)
	rjmp	memory_end
	
state_memory_send_dot:
        sbr     flags,(1<<dot_mem_bit)|(1<<part_of_char)
        rjmp   	memory_end

state_memory_send_dash:
        sbr     flags,(1<<dash_mem_bit)|(1<<part_of_char)
        rjmp    memory_end

state_memory_read_stop_all:	
	cbr	memory_flags,(1<<repeat_mode)
	
state_memory_read_stop:
	cbr	memory_flags,(1<<start_confirm)
	cbr	more_flags,(1<<memory_running)
	rcall	eeprom_clear_address
        cli
	ldi	data,1
	ldi	data_h,0
        mov	soft_timer_l,data
        mov	soft_timer_h,data_h
	mov	soft_timer_x,data_h
        sei
	sbrc	memory_flags,repeat_mode
	rjmp	state_memory_read_stop_repeat
	
	ldi	data,memory_repeat_wait_for_release
	rjmp	memory_set_new_state
	
state_memory_read_stop_repeat:	
	ldi	data,memory_repeat_start
	rjmp	memory_set_new_state
	
;............................................................

state_memory_repeat_start:
	lds	counter,ram_ptt_profile + repeat_time
	
	ldi	data,LOW(one_second)
	ldi	data_h,HIGH(one_second)
	
	mul	data,counter
	mov	timer_l,multiply_l
	mov	timer_h,multiply_h
	mul	data_h,counter
	add	timer_h,multiply_l
	clr	timer_x
					; higher bits in multiply_h ignored
	ldi	data,memory_repeat_run
	rjmp	memory_set_new_state
	
;............................................................

state_memory_repeat_run:
	sbrc	flags,memory_stop
	rjmp	state_memory_repeat_stop
	

	sbrs	sequencer_flags,timer_tic
	rjmp	memory_end
	
	rcall	decrement_timer
	_brne	memory_end

	ldi	data,memory_repeat_start_send
	rjmp	memory_set_new_state

state_memory_repeat_stop:
	cbr	memory_flags,(1<<repeat_mode)
	ldi	data,memory_repeat_wait_for_release
	rjmp	memory_set_new_state

;............................................................

state_memory_repeat_wait_for_release:
	sbrc	flags,memory_stop
	rjmp	memory_end

	ldi	data,memory_idle	
	rjmp	memory_set_new_state

;............................................................

state_memory_delay_send_start:
	ldi	data,LOW(one_second)
	ldi	data_h,HIGH(one_second)
	ldi	data_x,BYTE3(one_second)
	clc
	ror	data_x
	ror	data_h
	ror	data		; half second
	mov	timer_l,data
	mov	timer_h,data_h
	mov	timer_x,data_x
	ldi	data,memory_delay_send_run
	rjmp	memory_set_new_state
	
;............................................................

state_memory_delay_send_run:
	sbrs	sequencer_flags,timer_tic
	rjmp	memory_end
	
	rcall	decrement_timer
	_brne	memory_end

	ldi	data,memory_repeat_start_send
	rjmp	memory_set_new_state
	
;............................................................

state_memory_write_start:
        cbr     flags,((1<<dot_sent)|(1<<dash_sent)|(1<<pause_sent))
	clr	mem_bit_counter
	lds	data,memory_base_l
	lds	data_h,memory_base_h
	sts	memory_pointer_l,data
	sts	memory_pointer_h,data_h
	
	sbr	memory_flags,(1<<memory_write)
	ldi	data,memory_write_run
	rjmp	memory_set_new_state

;............................................................
	
state_memory_write_run:
	tst	memory_keys
	breq    memory_write_end_of_writing
	
        sbrc    flags,dot_sent
        rjmp    memory_write_dot

        sbrc    flags,dash_sent
        rjmp    memory_write_dash

        sbrc    flags,pause_sent
        rjmp    memory_write_pause

	rjmp    memory_end

memory_write_dot:
        ldi     data,1
        rjmp    memory_write_save

memory_write_dash:
        ldi     data,2
        rjmp    memory_write_save

memory_write_pause:
        ldi     data,0
;        rjmp    memory_write_save

memory_write_save:
	tst	mem_bit_counter
	brne	memory_write_save_ok

	ldi	data_h,4
	mov	mem_bit_counter,data_h
	
	lds	XL,memory_pointer_l
	lds	XH,memory_pointer_h
	adiw	X,1
	sts	memory_pointer_l,XL
	sts	memory_pointer_h,XH
	
	lds	YL,memory_end_l
	lds	YH,memory_end_h
	
	cp	XL,YL
	cpc	XH,YH
	breq	memory_write_end_of_memory
	
memory_write_save_ok:
        lsl     mem_data
        lsl     mem_data
        add     mem_data,data
        dec     mem_bit_counter
        _brne    memory_end

	mov	data,mem_data
	rcall	eeprom_write_byte
	rjmp	memory_end

memory_write_end_of_memory:
	ldi	data,memory_write_wait_for_release
	rjmp	memory_set_new_state
	
memory_write_end_of_writing:	
	ldi	data,memory_write_stop
	rjmp	memory_set_new_state

;............................................................

state_memory_write_wait_for_release:
	sbrc	flags,memory_stop
	rjmp	memory_end
	
	sbrc	memory_flags,command_memory
	sbr	memory_flags,(1<<analyse_command)
	
	ldi	data,memory_idle
	rjmp	memory_set_new_state
	
;............................................................

state_memory_write_stop:
	ldi	data,$FF
	tst	mem_bit_counter
	breq	memory_write_end_symbol
		
	ldi	data,3
	
memory_write_fill_byte_loop:
        lsl     mem_data
        lsl     mem_data
        add     mem_data,data
        dec     mem_bit_counter
	brne	memory_write_fill_byte_loop
	
	mov	data,mem_data
	rjmp	memory_write_last_byte
	
memory_write_end_symbol:	
	lds	XL,memory_pointer_l
	lds	XH,memory_pointer_h
	adiw	X,1
	lds	YL,memory_end_l
	lds	YH,memory_end_h
	cp	XL,YL
	cpc	XH,YH
	breq	memory_write_stop_end
	
memory_write_last_byte:	
	rcall	eeprom_write_byte
	
memory_write_stop_end:
	rcall	eeprom_clear_address
	cbr	memory_flags,(1<<memory_write)
	ldi	data,memory_write_wait_for_release
	rjmp	memory_set_new_state
		
;............................................................

memory_get_memory_addresses:	
	sts	memory_base_l,XL
	sts	memory_base_h,XH
				; get length of memory
	rcall	eeprom_get_byte
	clr	data_h
	adiw	X,1
	add	data,XL
	adc	data_h,XH
	sts	memory_end_l,data
	sts	memory_end_h,data_h
	ret
		
;............................................................

memory_set_new_state:
	sts	memory_state,data
	
memory_end:	
	ret
	 

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
